<?php
// $Id: channels_service.inc,v 1.2 2011/01/19 16:15:32 abouillis Exp $

/**
 * 
 * @file
 * The service interface
 *
 */
class ChannelsService extends ProcessApiServiceAbstract {
  
  protected static $DEBUG_MODE = TRUE;

 /**
   * Constructor for a service class, setting the server configuration used with
   * this service.
   *
   * @param ProcessApiServer $server
   *   The server object for this service.
   */
  /*public function __construct(ProcessApiServer $server) {
    $this->server = $server;
    $this->options = &$server->options;
  }/*/


  /**
   *
   *  Indicates if the debug mode is enabled on the given thread
   *
   * @param object $thread Thread being processed
   * @return bool
   *    TRUE if the debug mode is activated
   *    FALSE if the debug mode is not activated
   */
  public function debugModeEnabled($thread) {

    if(isset($thread->options['form']['channels_sync_options']['options']['debug']) && $thread->options['form']['channels_sync_options']['options']['debug'] == 1)
      return TRUE;

    return FALSE;
  }

  /**
   *
   *  Indicates if the force mode is enabled on the given thread
   *
   * @param object $thread Thread being processed
   * @return bool
   *    TRUE if the force mode is activated
   *    FALSE if the foce mode is not activated
   */
  public function forceDependenciesModeEnabled($thread) {

    if(isset($thread->options['form']['channels_sync_options']['options']['force_dependencies']) && $thread->options['form']['channels_sync_options']['options']['force_dependencies'] == 1)
      return TRUE;

    return FALSE;
  }


  /**
 * Return the dependencies for a given entity
 *
 * Exemple: node_reference, taxonomy_reference for a node
 *                parent terms ID for a taxonomy term
 *
 * @param int $entity_id Entity ID
 * @param string $entity_type Entity type
 * @return array Array of references, organized by type of reference
 */
  public static function getEntityDependencies($entity_id, $entity_type) {

    $all_dependencies = &drupal_static(__FUNCTION__, array());

    if(isset($all_dependencies[$entity_id])) {
      return $all_dependencies[$entity_id];
    }

    // load entity
   $entities = entity_load($entity_type, array($entity_id), array(), TRUE);

   if($entity_type == 'node') {

     $check_ext_references = FALSE;
     if(module_exists('ext_node_reference')) $check_ext_references = TRUE;

    $dependencies = array();

    if(!isset($entities[$entity_id])) return $dependencies;
    

    // load entity's fields list
    $all_fields_info = array_keys(field_info_instances($entity_type, $entities[$entity_id]->type));    

    // foreach each field
    foreach($all_fields_info as $field_name) {

      $field_info[$field_name] = field_info_field($field_name);

      // check if the current field type is a reference of the same type than the iniatial entity
      //if($field_info[$field_name]['type'] == $entity_type.'_reference') {

        if($field_info[$field_name]['type'] == 'taxonomy_term_reference'){
          //if(isset($item[$field_name.':tid']) && isset($item[$field_name.':tid']['value']) && is_array($item[$field_name.':tid']['value']))
            //$dependencies['taxonomy_term'][$field_name]  = $item[$field_name.':tid']['value'];
        }
        elseif( ($field_info[$field_name]['type'] == 'node_reference') || ($check_ext_references && $field_info[$field_name]['type'] == 'ext_node_reference') ){

          $field_values = field_get_items($entity_type, $entities[$entity_id], $field_name);

          if(is_array($field_values)) {
            foreach($field_values as $field_value_key => $field_value) {
              $dependencies['node'][] = $field_value['nid'];
            }
          }
        }
    }
   }
   elseif($entity_type == 'taxonomy_term') {

    $parent_terms = array_keys(taxonomy_get_parents ($entities[$entity_id]->tid));
    $dependencies = array('taxonomy_term' => $parent_terms);

    }

    if(isset($dependencies[$entity_type]))
      $all_dependencies[$entity_id][$entity_type] = $dependencies[$entity_type];

    return $dependencies;
}

  public function threadConfigurationForm(ProcessApiThread $thread, array $form, array &$form_state) {
  
    $form = parent::threadConfigurationForm($thread, $form, $form_state);
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // get the channels list
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    $channels = channels_get_channels();
    $channels_options = array('' => t('< Choose a channel >'));
    foreach ($channels as $channel)
      $channels_options[$channel->id] = $channel->name;

    $form['channels_channel'] = array(
        '#type' => 'select',
        '#title' => t('Channel'),
        '#description' => t('Select the channel to associate with this thread.'),
        '#default_value' => '',
        '#options' => $channels_options,
        '#required' => TRUE,
    );
    
    if (isset($form_state['values']['options']['form']['channels_services']['form']['channels_channel']))
      $form['channels_channel']['#default_value'] = $form_state['values']['options']['form']['channels_channel'];
    elseif (isset($thread->options['form']['channels_channel']))
        $form['channels_channel']['#default_value'] = $thread->options['form']['channels_channel'];
      
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // synchronisation options
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    $form['channels_sync_options'] = array(
      '#type'   => 'fieldset',
      '#title' => t('Synchronization options'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      'options' => array(
        'cascade' => array(
            '#type' => 'checkbox',
            '#title' => t('Cascade mode'),
            '#description' => t('Check to enable the cascade mode to allow the synchronization to follow references (node_reference for node, parent for taxonomy)'),
        ),
        'force_dependencies' => array(
            '#type' => 'checkbox',
            '#title' => t('Force dependencies synchronization'),
            '#description' => t('Check to enable the synchronization of a dependence which has not been declared on the channel'),
        ),
        'debug' => array(
            '#type' => 'checkbox',
            '#title' => t('Debug mode'),
            '#description' => t('Check to enable debug mode'),
        ),
      )
    );

    if (isset($form_state['values']['options']['form']['channels_sync_options']['options']['cascade']))
      $form['channels_sync_options']['options']['cascade']['#default_value'] = $form_state['values']['options']['form']['channels_sync_options']['options']['cascade'];
    elseif (isset($thread->options['form']['channels_sync_options']['options']['cascade']))
      $form['channels_sync_options']['options']['cascade']['#default_value'] = $thread->options['form']['channels_sync_options']['options']['cascade'];

    if (isset($form_state['values']['options']['form']['channels_sync_options']['options']['force_dependencies']))
      $form['channels_sync_options']['options']['force_dependencies']['#default_value'] = $form_state['values']['options']['form']['channels_sync_options']['options']['force_dependencies'];
    elseif (isset($thread->options['form']['channels_sync_options']['options']['force_dependencies']))
      $form['channels_sync_options']['options']['force_dependencies']['#default_value'] = $thread->options['form']['channels_sync_options']['options']['force_dependencies'];

    if (isset($form_state['values']['options']['form']['channels_sync_options']['options']['debug']))
      $form['channels_sync_options']['options']['debug']['#default_value'] = $form_state['values']['options']['form']['channels_sync_options']['options']['debug'];
    elseif (isset($thread->options['form']['channels_sync_options']['options']['debug']))
      $form['channels_sync_options']['options']['debug']['#default_value'] = $thread->options['form']['channels_sync_options']['options']['debug'];


    if($thread->entity_type == 'node') {
      $form['channels_sync_options']['options']['file_transfert'] = array(
              '#type' => 'checkbox',
              '#title' => t('File transfert enabled'),
              '#description' => t('Check to enable the file transfert'),
      );
      if (isset($form_state['values']['options']['form']['channels_sync_options']['options']['file_transfert']))
        $form['channels_sync_options']['options']['file_transfert']['#default_value'] = $form_state['values']['options']['form']['channels_sync_options']['options']['file_transfert'];
      elseif (isset($thread->options['form']['channels_sync_options']['options']['file_transfert']))
        $form['channels_sync_options']['options']['file_transfert']['#default_value'] = $thread->options['form']['channels_sync_options']['options']['file_transfert'];
    }

    

    /*if (isset($form_state['values']['options']['form']['channels_sync_options']['options']['forced']))
      $form['channels_sync_options']['options']['forced']['#default_value'] = $form_state['values']['options']['form']['channels_sync_options']['options']['forced'];
    elseif (isset($thread->options['form']['channels_sync_options']['options']['forced']))
      $form['channels_sync_options']['options']['forced']['#default_value'] = $thread->options['form']['channels_sync_options']['options']['forced'];*/

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // services section
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    $form['channels_services'] = array(
        '#type' => 'fieldset',
        '#title' => t('Publication services'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        'form' => array(
          'default' => array(
            '#type' => 'radios',
            '#title' => t('Default service'),
            '#description' => t('Select the default service to use on this thread.'),
            '#required' => TRUE,
          )
        ),
    );
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // list of services
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    $services = array();
 
    // no posted values
    if (!isset($form_state['values']['options']['form']['channels_services']['form']['services'])) {

      // look in database
      if (isset($thread->options['form']['channels_services']['form']['services']))
        foreach ($thread->options['form']['channels_services']['form']['services'] as $key => $value)
          $services[$key] = $value;
    }
    else {
      foreach ($form_state['values']['options']['form']['channels_services']['form']['services'] as $key => $value)
          $services[$key] = $value; 
    }  
    
    $weight = 0;  
    $form['channels_services']['form']['services'] = array();
    $service_list = channels_service_list();

    foreach ($service_list as $key => $value) {

      // create current service fieldset
      $form['channels_services']['form']['services'][$key] = array(
        '#type' => 'fieldset',
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        '#title' => $value['name'],
        'form' => array(
          'enabled' =>array(
            '#type' => 'checkbox',
            '#title' => t('Enabled'),
            '#weight' => 1,
            '#default_value' => isset($services[$key]['form']['enabled']) ? $services[$key]['form']['enabled'] : FALSE,
            ),
        ),
      );
      
      // add current service as an option in default service selector       
      $form['channels_services']['form']['default']['#options'][$key] = $value['name'];

      // load current service's settings form
      if (isset($service_list[$key]['class']) && class_exists($service_list[$key]['class']) ) {
        $service_class = $service_list[$key]['class'];       
        $service_obj = new $service_class($this->server);        
        $form['channels_services']['form']['services'][$key]['form']['settings'] = $service_obj->threadConfigurationForm($thread, $form, $form_state);
      }
      $form['channels_services']['form']['services'][$key]['form']['settings']['#weight'] = 2;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // default service value
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    
    if (isset($thread->options['form']['channels_services']['form']['default'])){
      $form['channels_services']['form']['default']['#default_value'] = $thread->options['form']['channels_services']['form']['default'];
    }
    elseif (isset($form_state['values']['options']['form']['channels_services']['form']['default']))
      $form['channels_services']['form']['default']['#default_value'] = $form_state['values']['options']['form']['channels_services']['form']['default'];
    else {
      $first_service = each($form['channels_services']['form']['default']['#options']);
      $form['channels_services']['form']['default']['#default_value'] = $first_service['key'];
    }
     
    return $form;
  }  


  /*public function threadAllowReprocessOnUpdate(ProcessApiThread $thread) {

    return FALSE;
  }*/

  /**
   * Default implementation.
   * @see ProcessApiServiceInterface::threadConfigurationFormSubmit()
   */ 
  public function threadConfigurationFormSubmit(ProcessApiThread $thread, array $form, array $values, array &$form_state) {
    if (!isset($thread->options['form'])) $thread->options['form']=array();
    $thread->options['form'] = array_merge($thread->options['form'], $values);
  }
  
  public function threadConfigurationFormValidate(ProcessApiThread $thread, array $form, array $values, array &$form_state) {
    if(isset($values['channels_services']['form']['services'])){
      foreach ($values['channels_services']['form']['services'] as $key => $service) {
        if($values['channels_services']['form']['default'] == $key && $service['form']['enabled']!=1){
          $default = $values['channels_services']['form']['default'];
          form_error($form['server_dependent']['options']['form']['channels_services']['form']['services'][$default], t('The selected default service must be enabled.'));
        }
      }
    }
  }

   /**
   * Process a single item.
   *
   * @param ProcessApiThread $thread
   * @param interger $id
   *   The main object id.
   * @param array $item
   *   The item to process.
   * @param array $context
   *   An array keyed with :
   *   - mode : The mode can be 'insert', 'update' or 'delete'.
   *   - entity : The top entity object.
   *
   * Array
   * (
   *     [<id>] => Array
   *         (
   *             [name]    => 'Field name',
   *             [value]   => 'Field value',
   *             [type]    => 'type',
   *             [wrapper] =>  wrapper object,
   *             // there is also one entry for each header defined by threadFieldsConfigurationHeaders()
   *             [<header_id>] => <conf value>
   *         ),
   *     ...
   * )
   *
   * @return boolean
   *  TRUE means entity is successfully processed
   *  FALSE means it's not and will be processed again
   */
  public function threadProcessItem(ProcessApiThread $thread, $id, array $item, $context) {

    $item = module_invoke_all('channels_service_preprocess_item', $id, $item, $context);

    self::$DEBUG_MODE = $this->debugModeEnabled($thread);
    if($thread->entity_type != 'node' && $thread->entity_type != 'taxonomy_term') return FALSE;
    
    self::debug('THREAD START: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));

    $channel = $thread->options['form']['channels_channel'];
    $default_service =  $thread->options['form']['channels_services']['form']['default'];
    $entities = entity_load($thread->entity_type,array($id));

    self::debug('!type !item: base processing on channel !channel', array('!type' => $thread->entity_type, '!item' => $id, '!channel' => $channel));

    if (channels_entity_is_supported($id, $thread->entity_type, $channel)) {

      // force the entity's dependencies to be sync
      if($this->forceDependenciesModeEnabled($thread)) {
        $dependencies = self::getEntityDependencies($id, $thread->entity_type);
        if(isset($dependencies[$thread->entity_type]) && count($dependencies[$thread->entity_type])>0) {
          foreach($dependencies[$thread->entity_type] as $key => $dep_id) {
            channels_entity_publish($thread->entity_type, $dep_id, $channel);
          }
        }
      }

      self::debug('!type !item: declared on the channel', array('!type' => $thread->entity_type, '!item' => $id));
      
      $data = array();

      // we need to process an empty item => load default service
      if(empty($item) && $context['mode'] == 'delete') {
        $service_info = channels_service_get_info($default_service);
        $service_obj = new $service_info['class']($this->server);
        
        if(!$service_obj->threadBeginProcess($thread)){
          self::debug('!type !item: begin processing failed', array('!type' => $thread->entity_type, '!item' => $id));
          self::debug('THREAD END: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));         
          module_invoke_all('channels_service_process_item_error', $channel, $thread->entity_type, $id);
          return FALSE;
        }

        //process_api_add_semaphore($thread->machine_name, $id);
        if(!$service_obj->threadPreprocessItem($thread, $id, $item, $context)) continue;

        $res = $service_obj->threadProcessItem($thread, $id, $item, $context);
        $errors = false;
        if( !$res ) module_invoke_all('channels_service_process_item_error', $channel, $thread->entity_type, $id);
        else module_invoke_all('channels_service_process_item_success', $channel, $thread->entity_type, $id);

        $service_obj->threadEndProcess($thread);

        //process_api_remove_semaphore($thread->machine_name, $id);
        
        if (!$res) {

          self::debug('THREAD START: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));

          return FALSE;
        }
        
      }
      // item is not empty => it contained fields and each of them has a service declared
      else {
        
        // group data by service
        foreach ($item as $key => $field) {
          unset($field['wrapper']);
          $data[$field['channels_service']][$id][$key] = $field;
          unset($data[$field['channels_service']][$id][$key]['channels_service']);          
        }
        
        $errors = 0;

        // run process by service
        foreach ($data as $service_name => $group) {

          if(!$thread->options['form']['channels_services']['form']['services'][$service_name]['form']['enabled'])
            continue;
          
          $service = ($service_name != $default_service) ? $service_name : $default_service;

          foreach ($group as $key => $item) {
              $group[$key] = $this->addPublicationOptions($id, $thread->entity_type, $item, $channel);            
            if(isset($group[$key]['channels_options']['value']['publish']) && $group[$key]['channels_options']['value']['publish'] != 1) {
              return FALSE;  
            }
          }
          
          $service_info = channels_service_get_info($service);
          $service_obj = new $service_info['class']($this->server);

          if(!$service_obj->threadBeginProcess($thread)){
            self::debug('!type !item: begin processing failed', array('!type' => $thread->entity_type, '!item' => $id));
            self::debug('THREAD END: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));
            module_invoke_all('channels_service_process_item_error', $channel, $thread->entity_type, $id);
            return FALSE;
          }
          
          foreach ($group as $key => $item) {
            
            //$item = $this->addPublicationOptions($id, $thread->entity_type, $item, $channel);

            if(!$service_obj->threadPreprocessItem($thread, $key, $item, $context)) continue;            
            $res = $service_obj->threadProcessItem($thread, $key, $item, $context);
            if( !$res ) $errors++;
            $service_obj->threadEndProcess($thread);           
          }
          
        }

        if ($errors > 0) {
          module_invoke_all('channels_service_process_item_error', $channel, $thread->entity_type, $id);

          self::debug('THREAD END: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));

          if(module_exists('process_api')) {
            process_api_add_semaphore($thread->entity_type, $id);
            $entity = current(entity_load($thread->entity_type, array($id)));
            entity_save($thread->entity_type, $entity);
            process_api_remove_semaphore($thread->entity_type, $id);
          }

          return FALSE;
        }
        else {
          module_invoke_all('channels_service_process_item_success', $channel, $thread->entity_type, $id);
          if(module_exists('process_api')) {
            process_api_add_semaphore($thread->entity_type, $id);
            $entity = current(entity_load($thread->entity_type, array($id)));
            entity_save($thread->entity_type, $entity);
            process_api_remove_semaphore($thread->entity_type, $id);
          }
        }

      }
    }
    else self::debug('!type !item: NOT declared on the channel', array('!type' => $thread->entity_type, '!item' => $id));

    self::debug('THREAD END: !machine_name ------------------------------------- ', array('!machine_name' => $thread->machine_name));

    return TRUE;
  }

  public function threadFieldsConfigurationHeaders(ProcessApiThread $thread) {
    return array('channels_service' => t('Channels\' service'),);
  }

  public function threadFieldConfigurationForm(ProcessApiThread $thread, $header_id, array $field_info, array $form, array &$form_state) {
    
    switch ($header_id) {
     
      case 'channels_service':
        
        if (isset($thread->options['form']['channels_services']['form']['services'])) {
        
          $options = array();
          $service_list =  channels_service_list();
          
          if (isset($thread->options['form']['channels_services']['form']['default']) && isset($service_list[$thread->options['form']['channels_services']['form']['default']]['name']))
            $options[$thread->options['form']['channels_services']['form']['default']] = $service_list[$thread->options['form']['channels_services']['form']['default']]['name'] . ' ' . t('(Default)');
          
          foreach ($thread->options['form']['channels_services']['form']['services'] as $key => $service) {
            
            if (isset($service_list[$key]['name']) && $service['form']['enabled']) {
              if (isset($thread->options['form']['channels_services']['form']['default']) && $key != $thread->options['form']['channels_services']['form']['default'])
                $options[$key] = $service_list[$key]['name'];
            }
          }
                    
          return array(
            '#type' => 'select',
            '#default_value' => isset($field_info['channels_service'])? $field_info['channels_service'] : $thread->options['form']['channels_services']['form']['default'],
            '#options' => $options,
          );
        }
                
      default:
        return array('#markup' => '&nbsp;');
    }
  }

/**
 *
 *  Add the eventual publication option to the item as fields
 *
 * @param int $id Item ID
 * @param string $type Item entity type
 * @param array $item Array representation of an item (fields)
 * @param int $channel_id Channel ID
 * @return array Altered item
 */
  protected function addPublicationOptions($id, $type, $item, $channel_id) {

    $channels = channels_get_channels_by_entity($id, $type);

    if(count($channels)>0) {
      $item['channels_options'] = array(
          'name' => t('Channels options'),
          'type' => 'list<array>',
      );

      if(isset($channels[$channel_id]) && is_array($channels[$channel_id]->options) && count($channels[$channel_id]->options)>0) {
        foreach($channels[$channel_id]->options as $opt_key => $opt_value) {
          $item['channels_options']['value'][$opt_key] = $opt_value;
        }
      }
    }

    return $item;
  }


  /**
   *
   *  Log a message during the process of a given thread on a specific channel
   *
   * @static
   * @param object $thread Thread being processed
   * @param int $id Entity ID
   * @param string $type Type of log
   * @param string $message Message to log
   *
   */
  public static function log($thread, $id, $type, $message) {

    $channel = $thread->options['form']['channels_channel'];
    $entities = entity_load($thread->entity_type, array($id));

    $bundle = 'unknown';

    if($id > 0 && isset($entities[$id])) {
      if( $thread->entity_type == 'node' ) $bundle = $entities[$id]->type;
      elseif( $thread->entity_type == 'taxonomy_term' ) $bundle = $entities[$id]->vocabulary_machine_name;
    }

    if(is_null($bundle) || $bundle == '') $bundle = 'NULL';

    channels_report_log ($channel, $type, $thread->entity_type, $bundle, $id, $thread->machine_name . ': ' . $message);
  }

  /**
   *
   *  Log a debug messagepublic class channel
   *
   * @static
   * @param string $message
   * @param array $variables
   */
  public static function debug($message, $variables = array()) {
    if(self::$DEBUG_MODE && !is_null($message)) watchdog('debug', $message, $variables);
  }

}

abstract class ChannelsServiceInstance {


  protected $supported_types =  array('node' => array(), 'taxonomy_term' => array() );

  protected static $MACHINE_NAME = 'core';

  protected $_error = NULL;

  /**
   * Return an error message
   *
   * @return string Error message
   */
  public function getErrorMessage() {
    return $this->_error;
  }
  
  
  /**
   * Return all supported entity types.
   * 
   * @return array
   */
  public function getSupportedEntityTypes() {
    $types = array();
    foreach ($this->supported_types as $type) $types[$type] = entity_get_info($type);
    return $types;
  }

  /**
   *
   *  Check if the entity is known as supported by the current service
   *
   * @param string $type Entity type (node, taxonomy_term..)
   * @param int $id Entity ID
   * @param int $channel_id Channel ID
   * @return bool TRUE if the given type is supported
   */
  public function entitySupported($type, $id, $channel_id) {

    $entities = entity_load($type, array($id));
    
    if(count($entities)==0) return FALSE;

    if( $type == 'node' ) $bundle = $entities[$id]->type;
    elseif( $type == 'taxonomy_term' ) $bundle = $entities[$id]->vocabulary_machine_name;
    else return FALSE;

    if(isset($this->supported_types[$channel_id][$type]) && in_array($bundle, $this->supported_types[$type])) {
      return TRUE;
    }
    elseif($this->typeIsSupported($type, $bundle)) {
      $this->registerTypeSupport($type, $bundle, $channel_id);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Register a couple type/bundle support for entity processing
   *
   * @param string $type Entity type
   * @param string $bundle Entity bundle name
   */
  public function registerTypeSupport($type, $bundle, $channel_id) {

    if(!isset($this->supported_types[$channel_id][$type])) $this->supported_types[$channel_id][$type] = array();

    if(!in_array($bundle, $this->supported_types[$channel_id][$type])) {
      $this->supported_types[$channel_id][$type][] = $bundle;
      self::debug('!type !bundle: registered on channel !channel', array('!channel' => $channel_id, '!type' => $type, '!bundle' => $bundle));
    }
  }

  /**
   *
   *  Indicates if the cascade mode synchronization is enabled on the given thread
   *
   * @param object $thread Thread being processed
   * @return bool
   *    TRUE if the cascade mode is activated
   *    FALSE if the cascade mode is not activated
   */
  public function cascadeModeEnabled($thread) {

    if(isset($thread->options['form']['channels_sync_options']['options']['cascade']) && $thread->options['form']['channels_sync_options']['options']['cascade'] == 1)
      return TRUE;

    return FALSE;
  }




  /**
   *
   *  Indicates if the file transfert synchronization is enabled on the given thread
   *
   * @param object $thread Thread being processed
   * @return bool
   *    TRUE if the file transfert is enabled
   *    FALSE if the fiel transfert is not enabled
   */
  public function fileTransfertEnabled($thread) {
    if(isset($thread->options['form']['channels_sync_options']['options']['file_transfert']) && $thread->options['form']['channels_sync_options']['options']['file_transfert'] == 1)
      return TRUE;

    return FALSE;
  }



    /**
   *
   *  Indicated if the forced mode synchronization is activated on the given thread
   *
   * @param object $thread Thread being processed
   * @return bool
   *    TRUE if the cascade mode is activated
   *    FALSE if the cascade mode is not activated
   */
  /*public function forcedModeActivated($thread) {

    if(isset($thread->options['form']['channels_sync_options']['options']['forced']) && $thread->options['form']['channels_sync_options']['options']['forced'] == 1)
      return TRUE;

    return FALSE;
  }*/



  /**
   *
   *  Log a message during the process of a given thread on a specific channel
   *
   * @static
   * @param object $thread Thread being processed
   * @param int $id Entity ID
   * @param string $type Type of log
   * @param string $message Message to log
   *
   */
  public static function log($thread, $id, $type, $message) {
    ChannelsService::log($thread, $id, $type, $message);
  }

  /**
   *
   *  Log a debug messagepublic class channel
   *
   * @static
   * @param string $message
   * @param array $variables
   */
  protected static function debug($message, $variables = array()) {
    ChannelsService::debug($message, $variables);
  }

  /**
   * Return a configuration form for the process api's thread,
   *
   * @param ProcessApiThread $thread
   *   The thread object
   *
   * @return array
   *   A form array for setting service-specific options.
   */
  abstract public function threadConfigurationForm(ProcessApiThread $thread, array $form, array &$form_state) ;

  /**
   * Allow process engines to do some init stuff.
   *
   * @param ProcessApiThread $thread
   * @return bool
   */
  public function threadBeginProcess(ProcessApiThread $thread) {}


  public function cleanItemFields($id, $type, $item) {

    $entities = entity_load($type, array($id));

    $fields = array();
    $fields = array_keys((array)$entities[$id]);
    
    foreach($item as $field_name => $field_value) {

      if($field_name == 'channels_options') continue;

      $res = preg_split('/:/', $field_name);
      
      if(count($res)>1) {
        if(!in_array($res[0], $fields)) {
        unset($item[$field_name]);
        }
      }
      elseif(!in_array($field_name, $fields)) {
        unset($item[$field_name]);
      }
    }

    unset($item['body']);

    return $item;
  }

  /**
   * Preprocesses an item on this thread.
   *
   * @param ProcessApiThread $thread
   * @param int $id Item ID
   * @param array $item Entity's fields to process.
   *
   * @param array $context
   *   An array keyed with :
   *   - mode : The mode can be 'insert', 'update' or 'delete'.
   *   - entity : The top entity object.
   *
   * @return boolean
   *  TRUE means the entity can be processed after preprocess
   *  FALSE means the entity can not be processed after preprocess
   */
  public function threadPreprocessItem(ProcessApiThread $thread, $id, array $item, $context) {

    self::debug('!type !item: preprocessing', array('!type' => $thread->entity_type, '!item' => $id));

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    // check item's dependencies
    ///////////////////////////////////////////////////////////////////////////////////////////////////////////
    if($this->cascadeModeEnabled($thread) && $context['mode'] != 'delete') {
      if($thread->entity_type == 'taxonomy_term') {

        // parent terms
        $parent_terms = taxonomy_get_parents($id);

        if(count($parent_terms)>0) {

          self::debug('postpone current taxonomy_term !item dependencies', array('!item' => $id));

          // if the parent term is postponed, the child is postponed too, to be processed after it
          if($this->postponeDependencies($thread, array_keys($parent_terms), $thread->entity_type)) {
            $entities = entity_load($thread->entity_type, array($id));
            //process_api_entity_update($entities[$id], $thread->entity_type);
            $thread->process(array($id => $entities[$id]));

            self::debug('!type !item postponed', array('!type' => $thread->entity_type, '!item' => $id));
          }
        }
      }
      elseif($thread->entity_type == 'node') {

        // get entity's dependencies and process them
        $dependencies  = ChannelsService::getEntityDependencies($id, $thread->entity_type);

        if(count($dependencies)>0) {
          self::debug('postpone current node !item dependencies', array('!item' => $id));
          $update_current_entity = FALSE;
          foreach($dependencies as $dep_type => $dep_ids) {
            if($this->postponeDependencies($thread, $dep_ids, $dep_type))
              $update_current_entity = TRUE;
          }
         /* if($update_current_entity) {
            $entities = entity_load($thread->entity_type, array($id), array(), TRUE);
            //process_api_entity_update($entities[$id], $thread->entity_type);
            //$thread->process(array($id => $entities[$id]));
            //sleep(1);
            //self::debug('!type !item postponed', array('!type' => $thread->entity_type, '!item' => $id));
          }*/
        }
      }
    }
    return TRUE;
  }

   /**
   * Process a single item.
   *
   * @param ProcessApiThread $thread
   * @param interger $id
   *   The main object id.
   * @param array $item
   *   The item to process.
   *
   * @param array $context
   *   An array keyed with :
   *   - mode : The mode can be 'insert', 'update' or 'delete'.
   *   - entity : The top entity object.
   *
   * Array
   * (
   *     [<id>] => Array
   *         (
   *             [name]    => 'Field name',
   *             [value]   => 'Field value',
   *             [type]    => 'type',
   *             [wrapper] =>  wrapper object,
   *             // there is also one entry for each header defined by threadFieldsConfigurationHeaders()
   *             [<header_id>] => <conf value>
   *         ),
   *     ...
   * )
   *
   * @return boolean
   *  TRUE means entity is successfully processed
   *  FALSE means it's not and will be processed again
   */
  public function threadProcessItem(ProcessApiThread $thread, $id, array $item, $context) {

    self::debug('!type !item: processing', array('!type' => $thread->entity_type, '!item' => $id));

    $item = $this->cleanItemFields($id, $thread->entity_type, $item);

    // check if a service configuration exists
    if( !isset($thread->options['form']['channels_services']['form']['services']) ) {
      self::log($thread, $id, 'error', t('Unable to process item: no service configuration found.'));
      return FALSE;
    }

    if( $context['mode'] == 'delete') {
      $res = $this->deleteEntity($thread->entity_type, $id);
    }
    // check if the current entity is supported by the remote service
    else{

      $channel = (int) $thread->options['form']['channels_channel'];

      if(!$this->entitySupported($thread->entity_type, $id, $channel)) {

        self::debug('!type !item: type is NOT supported', array('!type' => $thread->entity_type, '!item' => $id));

        $entities = entity_load($thread->entity_type, array($id));

        if($thread->entity_type == 'node') $bundle = $entities[$id]->type;
        elseif($thread->entity_type == 'taxonomy_term') $bundle = $entities[$id]->vocabulary_machine_name;
        else $bundle = 'unknown';

        //self::log($thread, $id, 'error', t('Unable to process item: bundle "!bundle" is not supported for type !type', array('!bundle' => $bundle, '!type' => $thread->entity_type)));
        return TRUE;
      }
      else self::debug('!type !item: type is supported', array('!type' => $thread->entity_type, '!item' => $id));

      // send the entity to the service
      $res = $this->sendEntity($thread->entity_type, $id, $item);

      // transfert file attachments if needed
      if($res && $this->fileTransfertEnabled($thread)){
        if(!$this->threadProcessItemAttachments($thread, $id, $item, $context))
          self::log($thread, $id, 'error', t('Unable to process item attachments'));
      }
    }
    if(!$res) self::log($thread, $id, 'error', t('Unable to process item, an error occured during synchronization: !error', array('!error' => $this->getErrorMessage())));
    self::debug('!type !item: process done ', array('!type' => $thread->entity_type, '!item' => $id));
    return $res;
  }

  
  public function threadProcessItemAttachments(ProcessApiThread $thread, $id, array $item, $context) {
    
    $error = FALSE;
    foreach($item as $key => $field) {

      if($field['type'] == 'file') {

        $key = preg_split('/:/', $key);

        $field2send = $field;
        unset($field2send['value']->rdf_mapping);
        
        if(!$this->sendAttachment($field2send['value'], $id, $key[0])){
          $error = TRUE;
          self::log ($thread, $id, 'error', t('Unable to send attachment for the field "!field_name"', array('!field_name' => $field2send['name'])));
        }
        else self::debug('!type !id: field !field -> attachments successfully sent', array('!type' => $thread->entity_type, '!id' => $id, '!field' => $field2send['name']));
      }
    }
    if(!$error) return TRUE;
    return FALSE;
  }
  
   /**
   * Allow process engines to do some cleanup stuff.
   *
   * @param ProcessApiThread $thread
   */
  public function threadEndProcess(ProcessApiThread $thread) {}


  /**
   * Postpone process for a list of entities
   * 
   * @param ProcessApiThread $thread
   * @param array $ids Array of entity ID
   * @param string $type
   *
   * @return Return TRUE if some entities has been postponed
   */
  protected function postponeDependencies($thread, $ids, $type) {

    $postponed = 0;

    $entities = entity_load($type, $ids);

    //$thread->process($entities);

    foreach($entities as $id => $entity) {

      self::debug('!type !item: is a dependency', array('!type' => $type, '!item' => $id));

      if(!$this->entityExists($id, $type)){
        
        //process_api_entity_update($entity, $type);
        $res = $thread->process(array($id =>$entities[$id]));
        
        if(in_array($id, $res))
          self::debug('!type !item: postponed', array('!type' => $type, '!item' => $id));
        else
          self::debug('!type !item: NOT postponed', array('!type' => $type, '!item' => $id));
        sleep(1);
        $postponed++;
      }
      else self::debug('!type !item: already exists on service', array('!type' => $type, '!item' => $id));
      
    }
    if($postponed>0) return TRUE;
    return FALSE;
  }


  /**
   *
   * Send an entity to the configured service in process api module
   *
   * @param string $type The type of entity (node, taxonomy_term,...)
   * @param int $id The ID of the entity
   * @param array $item The list of fields to save
   */
  abstract protected function sendEntity($type, $id, $item);


  /**
   *
   * Delete an entity to the configured service in process api module
   *
   * @param string $type The type of entity (node, taxonomy_term,...)
   * @param int $id The ID of the entity top delete
   * @return bool
   *  Return TRUE if the entity is successfully sent
   *  Return FALSE if an error occured
   */
  abstract protected function deleteEntity($type, $id);


  /**
   *
   *  Test if an entity already exists on remote service
   *
   * @param int $id Entity ID
   * @param string $type Entity type (node, taxonomy_term...)
   */
  abstract protected function entityExists($id, $type);

   /**
   *
   * Check if the given type is supported by the remote service
   *
   * @param string $type Entity type (node, taxonomy_term...)
   * @param string $bundle Entity bundle (article, page, ...)
   * @return bool
   *        TRUE if the type is supported
   *        FALSE if the type is not supported
   *
   */
  abstract protected function typeIsSupported($type, $bundle);


  /**
   * Send an entity attachment as a file to the service
   *
   * @param object $file Object representation of a file
   *
   * Example:
   * stdClass Object
   *  (
   *             [fid] => 18
   *             [uid] => 1
   *             [filename] =>
   *             [uri] => public://field/image/imagefield_nUslG8.png
   *             [filemime] => image/png
   *             [filesize] => 1303
   *             [status] => 1
   *             [timestamp] => 1295884715
   *  )
   * @param int $id Entity ID
   * @param string $field_name The field's name the file is attached
   * @return bool
   *  Return TRUE if the attachment has been successfully sent
   *  Return FALSE if an error occured during the attachment send
   *
   */
  abstract protected function sendAttachment($file, $id, $field_name);
  
}
